'''
dns_sprockets_version - Contains version for dns_sprockets.
-----------------------------------------------------------

.. Copyright (c) 2015 Neustar, Inc. All rights reserved.
.. See COPYRIGHT.txt for full notice.  See LICENSE.txt for terms and conditions.
'''

VERSION = '2.0.0'

# end of file
